@@ -246,7 +246,7 @@ namespace Sass {
     Expression_Obj val = 0;
 
     if (map) {
-      for (auto __key = (map->keys()).begin(); __key != (map->keys()).end(); ++__key) { Expression_Obj key = *(__key);
+      for (Expression_Obj key : map->keys()) {
         Expression_Obj value = map->at(key);
 
         if (variables.size() == 1) {
@@ -337,16 +337,15 @@ namespace Sass {
     // try to use generic function
     if (env->has("@warn[f]")) {
 
-      struct Sass_Callee callee = {
+      // add call stack entry
+      ctx.callee_stack.push_back({
         "@warn",
         w->pstate().path,
         w->pstate().line + 1,
         w->pstate().column + 1,
         SASS_CALLEE_FUNCTION,
         { env }
-      };
-      // add call stack entry
-      ctx.callee_stack.push_back(callee);
+      });
 
       Definition_Ptr def = Cast<Definition>((*env)["@warn[f]"]);
       // Block_Obj          body   = def->block();
@@ -385,16 +384,15 @@ namespace Sass {
     // try to use generic function
     if (env->has("@error[f]")) {
 
-      struct Sass_Callee callee = {
+      // add call stack entry
+      ctx.callee_stack.push_back({
         "@error",
         e->pstate().path,
         e->pstate().line + 1,
         e->pstate().column + 1,
         SASS_CALLEE_FUNCTION,
         { env }
-      };
-      // add call stack entry
-      ctx.callee_stack.push_back(callee);
+      });
 
       Definition_Ptr def = Cast<Definition>((*env)["@error[f]"]);
       // Block_Obj          body   = def->block();
@@ -430,16 +428,15 @@ namespace Sass {
     // try to use generic function
     if (env->has("@debug[f]")) {
 
-      struct Sass_Callee callee = {
+      // add call stack entry
+      ctx.callee_stack.push_back({
         "@debug",
         d->pstate().path,
         d->pstate().line + 1,
         d->pstate().column + 1,
         SASS_CALLEE_FUNCTION,
         { env }
-      };
-      // add call stack entry
-      ctx.callee_stack.push_back(callee);
+      });
 
       Definition_Ptr def = Cast<Definition>((*env)["@debug[f]"]);
       // Block_Obj          body   = def->block();
@@ -523,7 +520,7 @@ namespace Sass {
     Map_Obj mm = SASS_MEMORY_NEW(Map,
                                 m->pstate(),
                                 m->length());
-    for (auto __key = (m->keys()).begin(); __key != (m->keys()).end(); ++__key) { auto key = *(__key);
+    for (auto key : m->keys()) {
       Expression_Ptr ex_key = key->perform(this);
       Expression_Ptr ex_val = m->at(key);
       if (ex_val == NULL) continue;
@@ -814,6 +811,7 @@ namespace Sass {
       if (l_type == Expression::NUMBER && r_type == Expression::NUMBER) {
         Number_Ptr l_n = Cast<Number>(lhs);
         Number_Ptr r_n = Cast<Number>(rhs);
+        l_n->reduce(); r_n->reduce();
         rv = op_numbers(op_type, *l_n, *r_n, ctx.c_options, pstate);
       }
       else if (l_type == Expression::NUMBER && r_type == Expression::COLOR) {
@@ -947,7 +945,7 @@ namespace Sass {
     Env* env = environment();
     if (!env->has(full_name) || (!c->via_call() && Prelexer::re_special_fun(name.c_str()))) {
       if (!env->has("*[f]")) {
-        for (auto __arg = (args->elements()).begin(); __arg != (args->elements()).end(); ++__arg) { Argument_Obj arg = *(__arg);
+        for (Argument_Obj arg : args->elements()) {
           if (List_Obj ls = Cast<List>(arg->value())) {
             if (ls->size() == 0) error("() isn't a valid CSS value.", c->pstate());
           }
@@ -1010,15 +1008,14 @@ namespace Sass {
       bind(std::string("Function"), c->name(), params, args, &ctx, &fn_env, this);
       Backtrace here(backtrace(), c->pstate(), ", in function `" + c->name() + "`");
       exp.backtrace_stack.push_back(&here);
-      struct Sass_Callee callee = {
+      ctx.callee_stack.push_back({
         c->name().c_str(),
         c->pstate().path,
         c->pstate().line + 1,
         c->pstate().column + 1,
         SASS_CALLEE_FUNCTION,
         { env }
-      };
-      ctx.callee_stack.push_back(callee);
+      });
 
       // eval the body if user-defined or special, invoke underlying CPP function if native
       if (body /* && !Prelexer::re_special_fun(name.c_str()) */) {
@@ -1052,15 +1049,14 @@ namespace Sass {
 
       Backtrace here(backtrace(), c->pstate(), ", in function `" + c->name() + "`");
       exp.backtrace_stack.push_back(&here);
-      struct Sass_Callee callee = {
+      ctx.callee_stack.push_back({
         c->name().c_str(),
         c->pstate().path,
         c->pstate().line + 1,
         c->pstate().column + 1,
         SASS_CALLEE_C_FUNCTION,
         { env }
-      };
-      ctx.callee_stack.push_back(callee);
+      });
 
       To_C to_c;
       union Sass_Value* c_args = sass_make_list(params->length(), SASS_COMMA);
@@ -1146,7 +1142,7 @@ namespace Sass {
 
     if (Arguments_Ptr args = Cast<Arguments>(ex)) {
       List_Ptr ll = SASS_MEMORY_NEW(List, args->pstate(), 0, SASS_COMMA);
-      for (auto __arg = (args->elements()).begin(); __arg != (args->elements()).end(); ++__arg) { auto arg = *(__arg);
+      for(auto arg : args->elements()) {
         ll->append(arg->value());
       }
       ll->is_interpolant(args->is_interpolant());
@@ -1182,7 +1178,7 @@ namespace Sass {
       List_Obj ll = SASS_MEMORY_NEW(List, l->pstate(), 0, l->separator());
       // this fixes an issue with bourbon sample, not really sure why
       // if (l->size() && Cast<Null>((*l)[0])) { res += ""; }
-      for (auto __item = (l->elements()).begin(); __item != (l->elements()).end(); ++__item) { Expression_Obj item = *(__item);
+      for(Expression_Obj item : *l) {
         item->is_interpolant(l->is_interpolant());
         std::string rl(""); interpolation(ctx, rl, item, into_quotes, l->is_interpolant());
         bool is_null = Cast<Null>(item) != 0; // rl != ""
@@ -1499,24 +1495,26 @@ namespace Sass {
   {
     double lv = l.value();
     double rv = r.value();
+
     if (op == Sass_OP::DIV && rv == 0) {
       // XXX: this is never hit via spec tests
       return SASS_MEMORY_NEW(String_Quoted, pstate, lv ? "Infinity" : "NaN");
     }
+
     if (op == Sass_OP::MOD && !rv) {
       // XXX: this is never hit via spec tests
       throw Exception::ZeroDivisionError(l, r);
     }
 
-    size_t l_n_units = l.numerator_units().size();
-    size_t l_d_units = l.numerator_units().size();
-    size_t r_n_units = r.denominator_units().size();
-    size_t r_d_units = r.denominator_units().size();
+    size_t l_n_units = l.numerators.size();
+    size_t l_d_units = l.numerators.size();
+    size_t r_n_units = r.denominators.size();
+    size_t r_d_units = r.denominators.size();
     // optimize out the most common and simplest case
     if (l_n_units == r_n_units && l_d_units == r_d_units) {
       if (l_n_units + l_d_units <= 1 && r_n_units + r_d_units <= 1) {
-        if (l.numerator_units() == r.numerator_units()) {
-          if (l.denominator_units() == r.denominator_units()) {
+        if (l.numerators == r.numerators) {
+          if (l.denominators == r.denominators) {
             Number_Ptr v = SASS_MEMORY_COPY(&l);
             v->value(ops[op](lv, rv));
             return v;
@@ -1525,41 +1523,39 @@ namespace Sass {
       }
     }
 
-    Number tmp(&r); // copy
-    bool strict = op != Sass_OP::MUL && op != Sass_OP::DIV;
-    tmp.normalize(l.find_convertible_unit(), strict);
-    std::string l_unit(l.unit());
-    std::string r_unit(tmp.unit());
-    Number_Obj v = SASS_MEMORY_COPY(&l); // copy
-    v->pstate(pstate);
-    if (l_unit.empty() && (op == Sass_OP::ADD || op == Sass_OP::SUB || op == Sass_OP::MOD)) {
-      v->numerator_units() = r.numerator_units();
-      v->denominator_units() = r.denominator_units();
+    Number_Obj v = SASS_MEMORY_COPY(&l);
+    
+    if (l.is_unitless() && (op == Sass_OP::ADD || op == Sass_OP::SUB || op == Sass_OP::MOD)) {
+      v->numerators = r.numerators;
+      v->denominators = r.denominators;
     }
 
     if (op == Sass_OP::MUL) {
       v->value(ops[op](lv, rv));
-      for (size_t i = 0, S = r.numerator_units().size(); i < S; ++i) {
-        v->numerator_units().push_back(r.numerator_units()[i]);
-      }
-      for (size_t i = 0, S = r.denominator_units().size(); i < S; ++i) {
-        v->denominator_units().push_back(r.denominator_units()[i]);
-      }
+      v->numerators.insert(v->numerators.end(),
+        r.numerators.begin(), r.numerators.end()
+      );
+      v->denominators.insert(v->denominators.end(),
+        r.denominators.begin(), r.denominators.end()
+      );
     }
     else if (op == Sass_OP::DIV) {
       v->value(ops[op](lv, rv));
-      for (size_t i = 0, S = r.numerator_units().size(); i < S; ++i) {
-        v->denominator_units().push_back(r.numerator_units()[i]);
-      }
-      for (size_t i = 0, S = r.denominator_units().size(); i < S; ++i) {
-        v->numerator_units().push_back(r.denominator_units()[i]);
-      }
-    } else {
-      v->value(ops[op](lv, r.value() * r.convert_factor(l)));
-      // v->normalize();
-      return v.detach();
+      v->numerators.insert(v->numerators.end(),
+        r.denominators.begin(), r.denominators.end()
+      );
+      v->denominators.insert(v->denominators.end(),
+        r.numerators.begin(), r.numerators.end()
+      );
     }
-    v->normalize();
+    else {
+      Number ln(l), rn(r);
+      ln.reduce(); rn.reduce();
+      double f(rn.convert_factor(ln));
+      v->value(ops[op](lv, rn.value() * f));
+    }
+
+    v->pstate(pstate);
     return v.detach();
   }
 
