@@ -30,27 +30,27 @@
 #include "wincrypt.h"
 #endif
 
-#if defined __GNUC__ && ! defined __llvm__
-  #define GCC_VERSION (__GNUC__ * 10000 \
-                       + __GNUC_MINOR__ * 100 \
-                       + __GNUC_PATCHLEVEL__)
-  #if GCC_VERSION < 40500
-    #include <tr1/random>
-    #define IMPLEMENT_TR1
-    #define tr1ns std::tr1
-    #define uniform_real_distribution uniform_real
-  #else
-    #include <random>
-    #define tr1ns std
-  #endif
-#else
-  #include <random>
-  #define tr1ns std
-#endif
-
 #define ARG(argname, argtype) get_arg<argtype>(argname, env, sig, pstate, backtrace)
-#define ARGR(argname, argtype, lo, hi) get_arg_r(argname, env, sig, pstate, lo, hi, backtrace)
 #define ARGM(argname, argtype, ctx) get_arg_m(argname, env, sig, pstate, backtrace, ctx)
+#define ARGNR(argname) get_arg_nr(argname, env, sig, pstate, backtrace)
+
+// return a number object (copied since we want to have reduced units)
+#define ARGN(argname) get_arg_n(argname, env, sig, pstate, backtrace) // Number copy
+
+// special function for weird hsla percent (10px == 10% == 10 != 0.1)
+#define ARGVAL(argname) get_arg_val(argname, env, sig, pstate, backtrace) // double
+
+// macros for common ranges (u mean unsigned or upper, r for full range)
+#define DARG_U_FACT(argname) get_arg_r(argname, env, sig, pstate, backtrace, - 0.0, 1.0) // double
+#define DARG_R_FACT(argname) get_arg_r(argname, env, sig, pstate, backtrace, - 1.0, 1.0) // double
+#define DARG_U_BYTE(argname) get_arg_r(argname, env, sig, pstate, backtrace, - 0.0, 255.0) // double
+#define DARG_R_BYTE(argname) get_arg_r(argname, env, sig, pstate, backtrace, - 255.0, 255.0) // double
+#define DARG_U_PRCT(argname) get_arg_r(argname, env, sig, pstate, backtrace, - 0.0, 100.0) // double
+#define DARG_R_PRCT(argname) get_arg_r(argname, env, sig, pstate, backtrace, - 100.0, 100.0) // double
+
+// macros for color related inputs (rbg and alpha/opacity values)
+#define COLOR_NUM(argname) color_num(argname, env, sig, pstate, backtrace) // double
+#define ALPHA_NUM(argname) alpha_num(argname, env, sig, pstate, backtrace) // double
 
 namespace Sass {
   using std::stringstream;
@@ -102,6 +102,8 @@ namespace Sass {
 
   namespace Functions {
 
+    static Number tmpnr(ParserState("[FN]"), 0);
+
     inline void handle_utf8_error (const ParserState& pstate, Backtrace* backtrace)
     {
       try {
@@ -153,20 +155,88 @@ namespace Sass {
       return val;
     }
 
-    Number_Ptr get_arg_r(const std::string& argname, Env& env, Signature sig, ParserState pstate, double lo, double hi, Backtrace* backtrace)
+    double get_arg_r(const std::string& argname, Env& env, Signature sig, ParserState pstate, Backtrace* backtrace, double lo, double hi)
     {
       // Minimal error handling -- the expectation is that built-ins will be written correctly!
       Number_Ptr val = get_arg<Number>(argname, env, sig, pstate, backtrace);
-      double v = val->value();
+      tmpnr = val;
+      tmpnr.reduce();
+      double v = tmpnr.value();
       if (!(lo <= v && v <= hi)) {
         std::stringstream msg;
         msg << "argument `" << argname << "` of `" << sig << "` must be between ";
         msg << lo << " and " << hi;
         error(msg.str(), pstate, backtrace);
       }
+      return v;
+    }
+
+    const Number& get_arg_nr(const std::string& argname, Env& env, Signature sig, ParserState pstate, Backtrace* backtrace)
+    {
+      // Minimal error handling -- the expectation is that built-ins will be written correctly!
+      Number_Ptr val = get_arg<Number>(argname, env, sig, pstate, backtrace);
+      tmpnr = val;
+      tmpnr.reduce();
+      return tmpnr;
+    }
+
+    Number_Ptr get_arg_n(const std::string& argname, Env& env, Signature sig, ParserState pstate, Backtrace* backtrace)
+    {
+      // Minimal error handling -- the expectation is that built-ins will be written correctly!
+      Number_Ptr val = get_arg<Number>(argname, env, sig, pstate, backtrace);
+      val = SASS_MEMORY_COPY(val);
+      val->reduce();
       return val;
     }
 
+    double get_arg_v(const std::string& argname, Env& env, Signature sig, ParserState pstate, Backtrace* backtrace)
+    {
+      // Minimal error handling -- the expectation is that built-ins will be written correctly!
+      Number_Ptr val = get_arg<Number>(argname, env, sig, pstate, backtrace);
+      tmpnr = val;
+      tmpnr.reduce();
+      /*
+      if (tmpnr.unit() == "%") {
+        tmpnr.value(tmpnr.value() / 100);
+        tmpnr.numerators.clear();
+      } else {
+        if (!tmpnr.is_unitless()) error("argument " + argname + " of `" + std::string(sig) + "` must be unitless", pstate);
+      }
+      */
+      return tmpnr.value();
+    }
+
+    double get_arg_val(const std::string& argname, Env& env, Signature sig, ParserState pstate, Backtrace* backtrace)
+    {
+      // Minimal error handling -- the expectation is that built-ins will be written correctly!
+      Number_Ptr val = get_arg<Number>(argname, env, sig, pstate, backtrace);
+      tmpnr = val;
+      tmpnr.reduce();
+      return tmpnr.value();
+    }
+
+    double color_num(const std::string& argname, Env& env, Signature sig, ParserState pstate, Backtrace* backtrace)
+    {
+      Number_Ptr val = get_arg<Number>(argname, env, sig, pstate, backtrace);
+      tmpnr = val; tmpnr.reduce();
+      if (tmpnr.unit() == "%") {
+        return std::min(std::max(tmpnr.value() * 255 / 100.0, 0.0), 255.0);
+      } else {
+        return std::min(std::max(tmpnr.value(), 0.0), 255.0);
+      }
+    }
+
+
+    inline double alpha_num(const std::string& argname, Env& env, Signature sig, ParserState pstate, Backtrace* backtrace) {
+      Number_Ptr val = get_arg<Number>(argname, env, sig, pstate, backtrace);
+      tmpnr = val; tmpnr.reduce();
+      if (tmpnr.unit() == "%") {
+        return std::min(std::max(tmpnr.value(), 0.0), 100.0);
+      } else {
+        return std::min(std::max(tmpnr.value(), 0.0), 1.0);
+      }
+    }
+
     #define ARGSEL(argname, seltype, contextualize) get_arg_sel<seltype>(argname, env, sig, pstate, backtrace, ctx)
 
     template <typename T>
@@ -233,43 +303,9 @@ namespace Sass {
     // random_device degrades sharply once the entropy pool
     // is exhausted. For practical use, random_device is
     // generally only used to seed a PRNG such as mt19937.
-    static tr1ns::mt19937 rand(static_cast<unsigned int>(GetSeed()));
+    static std::mt19937 rand(static_cast<unsigned int>(GetSeed()));
 
-    tr1ns::uniform_real_distribution<> std_dist(0, 1);
-    #ifdef IMPLEMENT_TR1
-      tr1ns::variate_generator <
-        tr1ns::mt19937,
-        tr1ns::uniform_real_distribution <double>
-      > gen_std_dist(rand, std_dist);
-    #endif
-
-    // Using ULONG_MAX here seems to fail on Mac OSX Clang!?
-    tr1ns::uniform_real_distribution<> full_dist(0, 4294967296);
-    #ifdef IMPLEMENT_TR1
-      tr1ns::variate_generator <
-        tr1ns::mt19937,
-        tr1ns::uniform_real_distribution <double>
-      > gen_full_dist(rand, full_dist);
-    #endif
-
-    // helper function to retrieve a random number in interval
-    // works around some compiler issues with older gcc versions
-    static double random(double min, double max)
-    {
-      tr1ns::uniform_real_distribution<> distributor(min, max);
-      #ifdef IMPLEMENT_TR1
-        tr1ns::variate_generator <
-          tr1ns::mt19937,
-          tr1ns::uniform_real_distribution <>
-        > gen(rand, distributor);
-        distributor(rand);
-        return gen();
-      #else
-        return distributor(rand);
-      #endif
-    }
-
-    // supported features lookup table
+    // features
     static std::set<std::string> features {
       "global-variable-shadowing",
       "extend-selector-pseudoclass",
@@ -281,30 +317,14 @@ namespace Sass {
     // RGB FUNCTIONS
     ////////////////
 
-    inline double color_num(Number_Ptr n) {
-      if (n->unit() == "%") {
-        return std::min(std::max(n->value() * 255 / 100.0, 0.0), 255.0);
-      } else {
-        return std::min(std::max(n->value(), 0.0), 255.0);
-      }
-    }
-
-    inline double alpha_num(Number_Ptr n) {
-      if (n->unit() == "%") {
-        return std::min(std::max(n->value(), 0.0), 100.0);
-      } else {
-        return std::min(std::max(n->value(), 0.0), 1.0);
-      }
-    }
-
     Signature rgb_sig = "rgb($red, $green, $blue)";
     BUILT_IN(rgb)
     {
       return SASS_MEMORY_NEW(Color,
                              pstate,
-                             color_num(ARG("$red",   Number)),
-                             color_num(ARG("$green", Number)),
-                             color_num(ARG("$blue",  Number)));
+                             COLOR_NUM("$red"),
+                             COLOR_NUM("$green"),
+                             COLOR_NUM("$blue"));
     }
 
     Signature rgba_4_sig = "rgba($red, $green, $blue, $alpha)";
@@ -312,10 +332,10 @@ namespace Sass {
     {
       return SASS_MEMORY_NEW(Color,
                              pstate,
-                             color_num(ARG("$red",   Number)),
-                             color_num(ARG("$green", Number)),
-                             color_num(ARG("$blue",  Number)),
-                             alpha_num(ARG("$alpha", Number)));
+                             COLOR_NUM("$red"),
+                             COLOR_NUM("$green"),
+                             COLOR_NUM("$blue"),
+                             ALPHA_NUM("$alpha"));
     }
 
     Signature rgba_2_sig = "rgba($color, $alpha)";
@@ -323,7 +343,7 @@ namespace Sass {
     {
       Color_Ptr c_arg = ARG("$color", Color);
       Color_Ptr new_c = SASS_MEMORY_COPY(c_arg);
-      new_c->a(alpha_num(ARG("$alpha", Number)));
+      new_c->a(ALPHA_NUM("$alpha"));
       new_c->disp("");
       return new_c;
     }
@@ -340,14 +360,8 @@ namespace Sass {
     BUILT_IN(blue)
     { return SASS_MEMORY_NEW(Number, pstate, ARG("$color", Color)->b()); }
 
-    Signature mix_sig = "mix($color-1, $color-2, $weight: 50%)";
-    BUILT_IN(mix)
-    {
-      Color_Obj  color1 = ARG("$color-1", Color);
-      Color_Obj  color2 = ARG("$color-2", Color);
-      Number_Obj weight = ARGR("$weight", Number, 0, 100);
-
-      double p = weight->value()/100;
+    Color* colormix(Context& ctx, ParserState& pstate, Color* color1, Color* color2, double weight) {
+      double p = weight/100;
       double w = 2*p - 1;
       double a = color1->a() - color2->a();
 
@@ -362,6 +376,16 @@ namespace Sass {
                              color1->a()*p + color2->a()*(1-p));
     }
 
+    Signature mix_sig = "mix($color-1, $color-2, $weight: 50%)";
+    BUILT_IN(mix)
+    {
+      Color_Obj  color1 = ARG("$color-1", Color);
+      Color_Obj  color2 = ARG("$color-2", Color);
+      double weight = DARG_U_PRCT("$weight");
+      return colormix(ctx, pstate, color1, color2, weight);
+
+    }
+
     ////////////////
     // HSL FUNCTIONS
     ////////////////
@@ -447,9 +471,9 @@ namespace Sass {
     Signature hsl_sig = "hsl($hue, $saturation, $lightness)";
     BUILT_IN(hsl)
     {
-      return hsla_impl(ARG("$hue", Number)->value(),
-                       ARG("$saturation", Number)->value(),
-                       ARG("$lightness",  Number)->value(),
+      return hsla_impl(ARGVAL("$hue"),
+                       ARGVAL("$saturation"),
+                       ARGVAL("$lightness"),
                        1.0,
                        ctx,
                        pstate);
@@ -458,10 +482,10 @@ namespace Sass {
     Signature hsla_sig = "hsla($hue, $saturation, $lightness, $alpha)";
     BUILT_IN(hsla)
     {
-      return hsla_impl(ARG("$hue", Number)->value(),
-                       ARG("$saturation", Number)->value(),
-                       ARG("$lightness",  Number)->value(),
-                       ARG("$alpha",  Number)->value(),
+      return hsla_impl(ARGVAL("$hue"),
+                       ARGVAL("$saturation"),
+                       ARGVAL("$lightness"),
+                       ARGVAL("$alpha"),
                        ctx,
                        pstate);
     }
@@ -500,11 +524,11 @@ namespace Sass {
     BUILT_IN(adjust_hue)
     {
       Color_Ptr rgb_color = ARG("$color", Color);
-      Number_Ptr degrees = ARG("$degrees", Number);
+      double degrees = ARGVAL("$degrees");
       HSL hsl_color = rgb_to_hsl(rgb_color->r(),
                                  rgb_color->g(),
                                  rgb_color->b());
-      return hsla_impl(hsl_color.h + degrees->value(),
+      return hsla_impl(hsl_color.h + degrees,
                        hsl_color.s,
                        hsl_color.l,
                        rgb_color->a(),
@@ -516,7 +540,7 @@ namespace Sass {
     BUILT_IN(lighten)
     {
       Color_Ptr rgb_color = ARG("$color", Color);
-      Number_Ptr amount = ARGR("$amount", Number, 0, 100);
+      double amount = DARG_U_PRCT("$amount");
       HSL hsl_color = rgb_to_hsl(rgb_color->r(),
                                  rgb_color->g(),
                                  rgb_color->b());
@@ -528,7 +552,7 @@ namespace Sass {
 
       return hsla_impl(hsl_color.h,
                        hsl_color.s,
-                       hslcolorL + amount->value(),
+                       hslcolorL + amount,
                        rgb_color->a(),
                        ctx,
                        pstate);
@@ -538,7 +562,7 @@ namespace Sass {
     BUILT_IN(darken)
     {
       Color_Ptr rgb_color = ARG("$color", Color);
-      Number_Ptr amount = ARGR("$amount", Number, 0, 100);
+      double amount = DARG_U_PRCT("$amount");
       HSL hsl_color = rgb_to_hsl(rgb_color->r(),
                                  rgb_color->g(),
                                  rgb_color->b());
@@ -551,7 +575,7 @@ namespace Sass {
 
       return hsla_impl(hsl_color.h,
                        hsl_color.s,
-                       hslcolorL - amount->value(),
+                       hslcolorL - amount,
                        rgb_color->a(),
                        ctx,
                        pstate);
@@ -561,18 +585,17 @@ namespace Sass {
     BUILT_IN(saturate)
     {
       // CSS3 filter function overload: pass literal through directly
-      Number_Ptr amount = Cast<Number>(env["$amount"]);
-      if (!amount) {
+      if (!Cast<Number>(env["$amount"])) {
         return SASS_MEMORY_NEW(String_Quoted, pstate, "saturate(" + env["$color"]->to_string(ctx.c_options) + ")");
       }
 
-      ARGR("$amount", Number, 0, 100);
+      double amount = DARG_U_PRCT("$amount");
       Color_Ptr rgb_color = ARG("$color", Color);
       HSL hsl_color = rgb_to_hsl(rgb_color->r(),
                                  rgb_color->g(),
                                  rgb_color->b());
 
-      double hslcolorS = hsl_color.s + amount->value();
+      double hslcolorS = hsl_color.s + amount;
 
       // Saturation cannot be below 0 or above 100
       if (hslcolorS < 0) {
@@ -594,12 +617,12 @@ namespace Sass {
     BUILT_IN(desaturate)
     {
       Color_Ptr rgb_color = ARG("$color", Color);
-      Number_Ptr amount = ARGR("$amount", Number, 0, 100);
+      double amount = DARG_U_PRCT("$amount");
       HSL hsl_color = rgb_to_hsl(rgb_color->r(),
                                  rgb_color->g(),
                                  rgb_color->b());
 
-      double hslcolorS = hsl_color.s - amount->value();
+      double hslcolorS = hsl_color.s - amount;
 
       // Saturation cannot be below 0 or above 100
       if (hslcolorS <= 0) {
@@ -697,7 +720,7 @@ namespace Sass {
     BUILT_IN(opacify)
     {
       Color_Ptr color = ARG("$color", Color);
-      double amount = ARGR("$amount", Number, 0, 1)->value();
+      double amount = DARG_U_FACT("$amount");
       double alpha = std::min(color->a() + amount, 1.0);
       return SASS_MEMORY_NEW(Color,
                              pstate,
@@ -712,7 +735,7 @@ namespace Sass {
     BUILT_IN(transparentize)
     {
       Color_Ptr color = ARG("$color", Color);
-      double amount = ARGR("$amount", Number, 0, 1)->value();
+      double amount = DARG_U_FACT("$amount");
       double alpha = std::max(color->a() - amount, 0.0);
       return SASS_MEMORY_NEW(Color,
                              pstate,
@@ -745,10 +768,10 @@ namespace Sass {
         error("Cannot specify HSL and RGB values for a color at the same time for `adjust-color'", pstate);
       }
       if (rgb) {
-        double rr = r ? ARGR("$red",   Number, -255, 255)->value() : 0;
-        double gg = g ? ARGR("$green", Number, -255, 255)->value() : 0;
-        double bb = b ? ARGR("$blue",  Number, -255, 255)->value() : 0;
-        double aa = a ? ARGR("$alpha", Number, -1, 1)->value() : 0;
+        double rr = r ? DARG_R_BYTE("$red") : 0;
+        double gg = g ? DARG_R_BYTE("$green") : 0;
+        double bb = b ? DARG_R_BYTE("$blue") : 0;
+        double aa = a ? DARG_R_FACT("$alpha") : 0;
         return SASS_MEMORY_NEW(Color,
                                pstate,
                                color->r() + rr,
@@ -758,9 +781,9 @@ namespace Sass {
       }
       if (hsl) {
         HSL hsl_struct = rgb_to_hsl(color->r(), color->g(), color->b());
-        double ss = s ? ARGR("$saturation", Number, -100, 100)->value() : 0;
-        double ll = l ? ARGR("$lightness",  Number, -100, 100)->value() : 0;
-        double aa = a ? ARGR("$alpha",      Number, -1, 1)->value() : 0;
+        double ss = s ? DARG_R_PRCT("$saturation") : 0;
+        double ll = l ? DARG_R_PRCT("$lightness") : 0;
+        double aa = a ? DARG_R_FACT("$alpha") : 0;
         return hsla_impl(hsl_struct.h + (h ? h->value() : 0),
                          hsl_struct.s + ss,
                          hsl_struct.l + ll,
@@ -800,10 +823,10 @@ namespace Sass {
         error("Cannot specify HSL and RGB values for a color at the same time for `scale-color'", pstate);
       }
       if (rgb) {
-        double rscale = (r ? ARGR("$red",   Number, -100.0, 100.0)->value() : 0.0) / 100.0;
-        double gscale = (g ? ARGR("$green", Number, -100.0, 100.0)->value() : 0.0) / 100.0;
-        double bscale = (b ? ARGR("$blue",  Number, -100.0, 100.0)->value() : 0.0) / 100.0;
-        double ascale = (a ? ARGR("$alpha", Number, -100.0, 100.0)->value() : 0.0) / 100.0;
+        double rscale = (r ? DARG_R_PRCT("$red") : 0.0) / 100.0;
+        double gscale = (g ? DARG_R_PRCT("$green") : 0.0) / 100.0;
+        double bscale = (b ? DARG_R_PRCT("$blue") : 0.0) / 100.0;
+        double ascale = (a ? DARG_R_PRCT("$alpha") : 0.0) / 100.0;
         return SASS_MEMORY_NEW(Color,
                                pstate,
                                color->r() + rscale * (rscale > 0.0 ? 255 - color->r() : color->r()),
@@ -812,10 +835,10 @@ namespace Sass {
                                color->a() + ascale * (ascale > 0.0 ? 1.0 - color->a() : color->a()));
       }
       if (hsl) {
-        double hscale = (h ? ARGR("$hue",        Number, -100.0, 100.0)->value() : 0.0) / 100.0;
-        double sscale = (s ? ARGR("$saturation", Number, -100.0, 100.0)->value() : 0.0) / 100.0;
-        double lscale = (l ? ARGR("$lightness",  Number, -100.0, 100.0)->value() : 0.0) / 100.0;
-        double ascale = (a ? ARGR("$alpha",      Number, -100.0, 100.0)->value() : 0.0) / 100.0;
+        double hscale = (h ? DARG_R_PRCT("$hue") : 0.0) / 100.0;
+        double sscale = (s ? DARG_R_PRCT("$saturation") : 0.0) / 100.0;
+        double lscale = (l ? DARG_R_PRCT("$lightness") : 0.0) / 100.0;
+        double ascale = (a ? DARG_R_PRCT("$alpha") : 0.0) / 100.0;
         HSL hsl_struct = rgb_to_hsl(color->r(), color->g(), color->b());
         hsl_struct.h += hscale * (hscale > 0.0 ? 360.0 - hsl_struct.h : hsl_struct.h);
         hsl_struct.s += sscale * (sscale > 0.0 ? 100.0 - hsl_struct.s : hsl_struct.s);
@@ -824,7 +847,7 @@ namespace Sass {
         return hsla_impl(hsl_struct.h, hsl_struct.s, hsl_struct.l, alpha, ctx, pstate);
       }
       if (a) {
-        double ascale = (ARGR("$alpha", Number, -100.0, 100.0)->value()) / 100.0;
+        double ascale = (DARG_R_PRCT("$alpha")) / 100.0;
         return SASS_MEMORY_NEW(Color,
                                pstate,
                                color->r(),
@@ -858,21 +881,21 @@ namespace Sass {
       if (rgb) {
         return SASS_MEMORY_NEW(Color,
                                pstate,
-                               r ? ARGR("$red",   Number, 0, 255)->value() : color->r(),
-                               g ? ARGR("$green", Number, 0, 255)->value() : color->g(),
-                               b ? ARGR("$blue",  Number, 0, 255)->value() : color->b(),
-                               a ? ARGR("$alpha", Number, 0, 255)->value() : color->a());
+                               r ? DARG_U_BYTE("$red") : color->r(),
+                               g ? DARG_U_BYTE("$green") : color->g(),
+                               b ? DARG_U_BYTE("$blue") : color->b(),
+                               a ? DARG_U_BYTE("$alpha") : color->a());
       }
       if (hsl) {
         HSL hsl_struct = rgb_to_hsl(color->r(), color->g(), color->b());
         if (h) hsl_struct.h = std::fmod(h->value(), 360.0);
-        if (s) hsl_struct.s = ARGR("$saturation", Number, 0, 100)->value();
-        if (l) hsl_struct.l = ARGR("$lightness",  Number, 0, 100)->value();
-        double alpha = a ? ARGR("$alpha", Number, 0, 1.0)->value() : color->a();
+        if (s) hsl_struct.s = DARG_U_PRCT("$saturation");
+        if (l) hsl_struct.l = DARG_U_PRCT("$lightness");
+        double alpha = a ? DARG_U_FACT("$alpha") : color->a();
         return hsla_impl(hsl_struct.h, hsl_struct.s, hsl_struct.l, alpha, ctx, pstate);
       }
       if (a) {
-        double alpha = ARGR("$alpha", Number, 0, 1.0)->value();
+        double alpha = DARG_U_FACT("$alpha");
         return SASS_MEMORY_NEW(Color,
                                pstate,
                                color->r(),
@@ -989,8 +1012,7 @@ namespace Sass {
         String_Constant_Ptr i = ARG("$insert", String_Constant);
         std::string ins = i->value();
         ins = unquote(ins);
-        Number_Ptr ind = ARG("$index", Number);
-        double index = ind->value();
+        double index = ARGVAL("$index");
         size_t len = UTF_8::code_point_count(str, 0, str.size());
 
         if (index > 0 && index <= len) {
@@ -1055,8 +1077,8 @@ namespace Sass {
       std::string newstr;
       try {
         String_Constant_Ptr s = ARG("$string", String_Constant);
-        double start_at = ARG("$start-at", Number)->value();
-        double end_at = ARG("$end-at", Number)->value();
+        double start_at = ARGVAL("$start-at");
+        double end_at = ARGVAL("$end-at");
         String_Quoted_Ptr ss = Cast<String_Quoted>(s);
 
         std::string str = unquote(s->value());
@@ -1150,7 +1172,7 @@ namespace Sass {
     Signature percentage_sig = "percentage($number)";
     BUILT_IN(percentage)
     {
-      Number_Ptr n = ARG("$number", Number);
+      Number_Obj n = ARGN("$number");
       if (!n->is_unitless()) error("argument $number of `" + std::string(sig) + "` must be unitless", pstate);
       return SASS_MEMORY_NEW(Number, pstate, n->value() * 100, "%");
     }
@@ -1158,41 +1180,37 @@ namespace Sass {
     Signature round_sig = "round($number)";
     BUILT_IN(round)
     {
-      Number_Ptr n = ARG("$number", Number);
-      Number_Ptr r = SASS_MEMORY_COPY(n);
-      r->pstate(pstate);
+      Number_Obj r = ARGN("$number");
       r->value(Sass::round(r->value(), ctx.c_options.precision));
-      return r;
+      r->pstate(pstate);
+      return r.detach();
     }
 
     Signature ceil_sig = "ceil($number)";
     BUILT_IN(ceil)
     {
-      Number_Ptr n = ARG("$number", Number);
-      Number_Ptr r = SASS_MEMORY_COPY(n);
-      r->pstate(pstate);
+      Number_Obj r = ARGN("$number");
       r->value(std::ceil(r->value()));
-      return r;
+      r->pstate(pstate);
+      return r.detach();
     }
 
     Signature floor_sig = "floor($number)";
     BUILT_IN(floor)
     {
-      Number_Ptr n = ARG("$number", Number);
-      Number_Ptr r = SASS_MEMORY_COPY(n);
-      r->pstate(pstate);
+      Number_Obj r = ARGN("$number");
       r->value(std::floor(r->value()));
-      return r;
+      r->pstate(pstate);
+      return r.detach();
     }
 
     Signature abs_sig = "abs($number)";
     BUILT_IN(abs)
     {
-      Number_Ptr n = ARG("$number", Number);
-      Number_Ptr r = SASS_MEMORY_COPY(n);
-      r->pstate(pstate);
+      Number_Obj r = ARGN("$number");
       r->value(std::abs(r->value()));
-      return r;
+      r->pstate(pstate);
+      return r.detach();
     }
 
     Signature min_sig = "min($numbers...)";
@@ -1251,19 +1269,13 @@ namespace Sass {
           err << "Expected $limit to be an integer but got " << lv << " for `random'";
           error(err.str(), pstate);
         }
-        // std::uniform_real_distribution<> distributor(1, lv + 1);
-        // uint_fast32_t distributed = static_cast<uint_fast32_t>(distributor(rand));
-        uint_fast32_t distributed = random(1, lv + 1);
+        std::uniform_real_distribution<> distributor(1, lv + 1);
+        uint_fast32_t distributed = static_cast<uint_fast32_t>(distributor(rand));
         return SASS_MEMORY_NEW(Number, pstate, (double)distributed);
       }
       else if (b) {
-        // std::uniform_real_distribution<> distributor(0, 1);
-        // double distributed = static_cast<double>(distributor(rand));
-        #ifdef IMPLEMENT_TR1
-          double distributed = gen_std_dist();
-        #else
-          double distributed = std_dist(rand);
-        #endif
+        std::uniform_real_distribution<> distributor(0, 1);
+        double distributed = static_cast<double>(distributor(rand));
         return SASS_MEMORY_NEW(Number, pstate, distributed);
       } else if (v) {
         throw Exception::InvalidArgumentType(pstate, "random", "$limit", "number", v);
@@ -1306,20 +1318,20 @@ namespace Sass {
     Signature nth_sig = "nth($list, $n)";
     BUILT_IN(nth)
     {
-      Number_Ptr n = ARG("$n", Number);
+      double nr = ARGVAL("$n");
       Map_Ptr m = Cast<Map>(env["$list"]);
       if (Selector_List_Ptr sl = Cast<Selector_List>(env["$list"])) {
         size_t len = m ? m->length() : sl->length();
         bool empty = m ? m->empty() : sl->empty();
         if (empty) error("argument `$list` of `" + std::string(sig) + "` must not be empty", pstate);
-        double index = std::floor(n->value() < 0 ? len + n->value() : n->value() - 1);
+        double index = std::floor(nr < 0 ? len + nr : nr - 1);
         if (index < 0 || index > len - 1) error("index out of bounds for `" + std::string(sig) + "`", pstate);
         // return (*sl)[static_cast<int>(index)];
         Listize listize;
         return (*sl)[static_cast<int>(index)]->perform(&listize);
       }
       List_Obj l = Cast<List>(env["$list"]);
-      if (n->value() == 0) error("argument `$n` of `" + std::string(sig) + "` must be non-zero", pstate);
+      if (nr == 0) error("argument `$n` of `" + std::string(sig) + "` must be non-zero", pstate);
       // if the argument isn't a list, then wrap it in a singleton list
       if (!m && !l) {
         l = SASS_MEMORY_NEW(List, pstate, 1);
@@ -1328,7 +1340,7 @@ namespace Sass {
       size_t len = m ? m->length() : l->length();
       bool empty = m ? m->empty() : l->empty();
       if (empty) error("argument `$list` of `" + std::string(sig) + "` must not be empty", pstate);
-      double index = std::floor(n->value() < 0 ? len + n->value() : n->value() - 1);
+      double index = std::floor(nr < 0 ? len + nr : nr - 1);
       if (index < 0 || index > len - 1) error("index out of bounds for `" + std::string(sig) + "`", pstate);
 
       if (m) {
@@ -1546,7 +1558,7 @@ namespace Sass {
     {
       Map_Obj m = ARGM("$map", Map, ctx);
       List_Ptr result = SASS_MEMORY_NEW(List, pstate, m->length(), SASS_COMMA);
-      for (auto __key = (m->keys()).begin(); __key != (m->keys()).end(); ++__key) { auto key = *(__key);
+      for ( auto key : m->keys()) {
         result->append(key);
       }
       return result;
@@ -1557,7 +1569,7 @@ namespace Sass {
     {
       Map_Obj m = ARGM("$map", Map, ctx);
       List_Ptr result = SASS_MEMORY_NEW(List, pstate, m->length(), SASS_COMMA);
-      for (auto __key = (m->keys()).begin(); __key != (m->keys()).end(); ++__key) { auto key = *(__key);
+      for ( auto key : m->keys()) {
         result->append(m->at(key));
       }
       return result;
@@ -1584,7 +1596,7 @@ namespace Sass {
       Map_Obj m = ARGM("$map", Map, ctx);
       List_Obj arglist = ARG("$keys", List);
       Map_Ptr result = SASS_MEMORY_NEW(Map, pstate, 1);
-      for (auto __key = (m->keys()).begin(); __key != (m->keys()).end(); ++__key) { auto key = *(__key);
+      for (auto key : m->keys()) {
         remove = false;
         for (size_t j = 0, K = arglist->length(); j < K && !remove; ++j) {
           remove = Eval::eq(key, arglist->value_at_index(j));
@@ -1624,23 +1636,33 @@ namespace Sass {
 
     Signature unit_sig = "unit($number)";
     BUILT_IN(unit)
-    { return SASS_MEMORY_NEW(String_Quoted, pstate, quote(ARG("$number", Number)->unit(), '"')); }
+    { 
+      Number_Obj arg = ARGN("$number");
+      std::string str(quote(arg->unit(), '"'));
+      return SASS_MEMORY_NEW(String_Quoted, pstate, str);
+    }
 
     Signature unitless_sig = "unitless($number)";
     BUILT_IN(unitless)
-    { return SASS_MEMORY_NEW(Boolean, pstate, ARG("$number", Number)->is_unitless()); }
+    {
+      Number_Obj arg = ARGN("$number");
+      bool unitless = arg->is_unitless();
+      return SASS_MEMORY_NEW(Boolean, pstate, unitless);
+    }
 
     Signature comparable_sig = "comparable($number-1, $number-2)";
     BUILT_IN(comparable)
     {
-      Number_Ptr n1 = ARG("$number-1", Number);
-      Number_Ptr n2 = ARG("$number-2", Number);
+      Number_Obj n1 = ARGN("$number-1");
+      Number_Obj n2 = ARGN("$number-2");
       if (n1->is_unitless() || n2->is_unitless()) {
         return SASS_MEMORY_NEW(Boolean, pstate, true);
       }
-      Number tmp_n2(n2); // copy
-      tmp_n2.normalize(n1->find_convertible_unit());
-      return SASS_MEMORY_NEW(Boolean, pstate, n1->unit() == tmp_n2.unit());
+      // normalize into main units
+      n1->normalize(); n2->normalize();
+      Units &lhs_unit = *n1, &rhs_unit = *n2;
+      bool is_comparable = (lhs_unit == rhs_unit);
+      return SASS_MEMORY_NEW(Boolean, pstate, is_comparable);
     }
 
     Signature variable_exists_sig = "variable-exists($name)";
@@ -2031,11 +2053,8 @@ namespace Sass {
     BUILT_IN(unique_id)
     {
       std::stringstream ss;
-      #ifdef IMPLEMENT_TR1
-        uint_fast32_t distributed = gen_full_dist();
-      #else
-        uint_fast32_t distributed = full_dist(rand);
-      #endif
+      std::uniform_real_distribution<> distributor(0, 4294967296); // 16^8
+      uint_fast32_t distributed = static_cast<uint_fast32_t>(distributor(rand));
       ss << "u" << std::setfill('0') << std::setw(8) << std::hex << distributed;
       return SASS_MEMORY_NEW(String_Quoted, pstate, ss.str());
     }
