@@ -352,7 +352,7 @@ namespace Sass {
     if (map->is_invisible()) return;
     bool items_output = false;
     append_string("(");
-    for (auto __key = (map->keys()).begin(); __key != (map->keys()).end(); ++__key) { auto key = *(__key);
+    for (auto key : map->keys()) {
       if (items_output) append_comma_separator();
       key->perform(this);
       append_colon_separator();
@@ -505,6 +505,9 @@ namespace Sass {
 
     std::string res;
 
+    // reduce units
+    n->reduce();
+
     // check if the fractional part of the value equals to zero
     // neat trick from http://stackoverflow.com/a/1521682/1550314
     // double int_part; bool is_int = modf(value, &int_part) == 0.0;
@@ -655,7 +658,7 @@ namespace Sass {
       ss << name;
     }
     else if (r == 0 && g == 0 && b == 0 && a == 0) {
-        ss << "transparent";
+      ss << "transparent";
     }
     else if (a >= 1) {
       if (res_name != "") {
