@@ -65,7 +65,7 @@ namespace Sass {
     }
 
     // search for unicode char
-    for (auto __chr = (wbuf.buffer).begin(); __chr != (wbuf.buffer).end(); ++__chr) { const char& chr = *(__chr);
+    for(const char& chr : wbuf.buffer) {
       // skip all ascii chars
       // static cast to unsigned to handle `char` being signed / unsigned
       if (static_cast<unsigned>(chr) < 128) continue;
@@ -134,6 +134,7 @@ namespace Sass {
       append_string(ss.str());
       append_optional_linefeed();
     }
+    scheduled_crutch = s;
     if (s) s->perform(this);
     append_scope_opener(b);
     for (size_t i = 0, L = b->length(); i < L; ++i) {
