@@ -35,7 +35,7 @@ namespace Sass {
   }
 
   /* Locale unspecific atof function. */
-  double sass_atof(const char *str)
+  double sass_strtod(const char *str)
   {
     char separator = *(localeconv()->decimal_point);
     if(separator != '.'){
@@ -48,13 +48,13 @@ namespace Sass {
         // of the string. This is slower but it is thread safe.
         char *copy = sass_copy_c_string(str);
         *(copy + (found - str)) = separator;
-        double res = atof(copy);
+        double res = strtod(copy, NULL);
         free(copy);
         return res;
       }
     }
 
-    return atof(str);
+    return strtod(str, NULL);
   }
 
   // helper for safe access to c_ctx
@@ -100,7 +100,7 @@ namespace Sass {
   {
     std::string out("");
     bool esc = false;
-    for (auto __i = (str).begin(); __i != (str).end(); ++__i) { auto i = *(__i);
+    for (auto i : str) {
       if (i == '\\') {
         esc = ! esc;
       } else if (esc && i == '\r') {
@@ -127,7 +127,7 @@ namespace Sass {
   {
     std::string out("");
     bool esc = false;
-    for (auto __i = (str).begin(); __i != (str).end(); ++__i) { auto i = *(__i);
+    for (auto i : str) {
       if (i == '\\' && !esc) {
         out += '\\';
         out += '\\';
@@ -168,7 +168,7 @@ namespace Sass {
   {
     std::string out("");
     bool lf = false;
-    for (auto __i = (str).begin(); __i != (str).end(); ++__i) { auto i = *(__i);
+    for (auto i : str) {
       if (i == '\n') {
         out += ' ';
         lf = true;
@@ -186,7 +186,7 @@ namespace Sass {
     size_t has = 0;
     char prev = 0;
     bool clean = false;
-    for (auto __i = (text).begin(); __i != (text).end(); ++__i) { auto i = *(__i);
+    for (auto i : text) {
       if (clean) {
         if (i == '\n') { has = 0; }
         else if (i == '\r') { has = 0; }
@@ -600,8 +600,10 @@ namespace Sass {
         }
         else if (Has_Block_Ptr b = Cast<Has_Block>(stm)) {
           Block_Obj pChildBlock = b->block();
-          if (isPrintable(pChildBlock, style)) {
-            hasPrintableChildBlocks = true;
+          if (!b->is_invisible()) {
+            if (isPrintable(pChildBlock, style)) {
+              hasPrintableChildBlocks = true;
+            }
           }
         }
 
